<?php

// $Id: SmartyAggregate.php,v 1.1.2.5 2010/06/21 18:28:37 fgiasson Exp $

include_once("smarty/Smarty.class.php");
include_once("WebServiceQuerier.php");


/*! @ingroup ConStructModule */
//@{

/*! @file SmartyAggregate.php
   @brief Smarty API class that aggregate triples of resources using filters on types, properties and values.
  
   \n\n
 
   @author Frederick Giasson, Structured Dynamics LLC.

   \n\n\n
 */

/*!	 @brief Smarty API class that aggregate triples of resources using filters on types, properties and values.
     @details This class is used to aggregate all information needed to display list of things, in a template, using some primitif filtering conditions on types, properties and values. These filters creates a SPARQL query that will be sent to the triple store to aggregate all the triples that describe resources that comply to the filtering conditions.
            
    \n
    
    @todo This method could be changed by Drupal Views 2 in the future; if CCK is properly linked to the RDF World.
    
    @author Frederick Giasson, Structured Dynamics LLC.
  
    \n\n\n
*/
class SmartyAggregate
{
  /*! @brief Array of instance records that complies to the filtering conditions */
  private $instanceRecords;

  /*! @brief Dataset where the records are indexed */
  private $dataset;

  function __construct($dataset)
  {
    $this->dataset = $dataset;
  }

/*!	 @brief Get an aggregate request and populate the instance records array according to the filter condition
            
    \n
    
    @param[in] $params Smarty parameters that come from the template definition. Parameters looks like: "t1, t2, t3, t*", "p1, p2, p3, p*", "v1, v2, v3, v*" where T stands for Type, P stands for Property and V stands for Values. Filters can be created with one or more type and property. Ad we can use an optional value for the property.
    @param[in] $smarty_obj reference to the smarty object being processed
    
    @return returns the literal that refers to that label
  
    @warning A special care as to be taken when the user create the filters. Bad filters on big datasets can lead to terrible performances.
  
    @author Frederick Giasson, Structured Dynamics LLC.
  
    \n\n\n
*/
  public function getAggregate($params, &$smarty_obj)
  {
    $nbTypeConstraints = 0;
    $nbPropertyConstraints = 0;
    $nbValueConstraints = 0;

    // Count the number of type constraints.
    for ($i = 1; $i < 64; $i++)
    {
      if (isset($params["t" . $i]))
      {
        $nbTypeConstraints++;
      }

      if (isset($params["p" . $i]))
      {
        $nbPropertyConstraints++;
      }

      if (isset($params["v" . $i]))
      {
        $nbValueConstraints++;
      }
    }

    // Build the SPARQL query by using these constraints.

    $query = "select ?s ?p ?o where {";

    if ($nbTypeConstraints > 0)
    {
      for ($i = 1; $i <= $nbTypeConstraints; $i++)
      {
        $query .= "?s a <" . get_label_uri($params["t" . $i]) . ">.";
      }
    }

    $filters = "";

    if ($nbPropertyConstraints > 0)
    {
      for ($i = 1; $i <= $nbPropertyConstraints; $i++)
      {
        $query .= "?s <" . get_label_uri($params["p" . $i]) . "> ?filter$i.";

        if (isset($params["v" . $i]) && $params["v" . $i] != "")
        {
          $filters .= "filter(?filter$i = '" . $params["v" . $i] . "').";
        }
      }
    }
               
    $query .= "?s ?p ?o." . $filters . "}";
          
    if ($nbTypeConstraints > 0 || $nbPropertyConstraints > 0)
    {
      /*
        @TODO try to get related WSF endpoint instead of "localhost" only.
      */
      $wsq = new WebServiceQuerier("http://localhost/ws/sparql/", "post", "application/sparql-results+xml",
        "query=" . urlencode($query) . "&dataset=" . urlencode($this->dataset) . "&registered_ip=self", 5);

      if ($wsq->getStatus() == 200)
      {
        $xml = xml2ary($wsq->getResultset());

        if (isset($xml["sparql"]["_c"]["results"]["_c"]["result"]))
        {
          foreach ($xml["sparql"]["_c"]["results"]["_c"]["result"] as $result)
          {
            $s = "";
            $p = "";
            $o = "";

            foreach ($result["_c"]["binding"] as $binding)
            {
              $boundVariable = $binding["_a"]["name"];

              $keys = array_keys($binding["_c"]);

              $boundType = $keys[0];
              $boundValue = $binding["_c"][$boundType]["_v"];

              switch ($boundVariable)
              {
                case "s":
                  $s = $boundValue;
                  break;

                case "p":
                  $p = $boundValue;
                  break;

                case "o":
                  $o = $boundValue;
                  break;
              }
            }

            if (!isset($this->instanceRecords[$s][$p]))
            {
              $this->instanceRecords[$s][$p] = array($o);
            }
            else
            {
              array_push($this->instanceRecords[$s][$p], $o);
            }
          }
        }
      }
    }
  }

  /*!	 @brief Get the list of all instance records retreived after all aggregates calls
              
      \n
      
      @param[in] $params No parameters expected
      @param[in] $smarty_obj reference to the smarty object being processed
      
      @return returns the list of all instance records and their description
    
      @author Frederick Giasson, Structured Dynamics LLC.
    
      \n\n\n
  */
  public function getInstanceRecords($params, &$smarty_obj)
  {
    $instanceRecords = $this->instanceRecords;

    // Sort the instanceRecords array by the first value of the target property
    if (isset($params["orderby"]))
    {
      // Case insensitive ordering
      foreach ($instanceRecords as $propertiesValues)
      {
        $sort_aux[] = strtolower($propertiesValues[get_label_uri($params["orderby"])][0]);
      }

      array_multisort($sort_aux, SORT_ASC, SORT_STRING, $instanceRecords);
    }

    return ($instanceRecords);
  }

  /*!	 @brief Get the list of all instance records that have a speciifc property and value
              
      \n
      
      @param[in] $params "property" and "value" being the tagetted property and value. "orderby" the property to order by 
      @param[in] $smarty_obj reference to the smarty object being processed
      
      @return returns the list of all instance records and their description
    
      @author Frederick Giasson, Structured Dynamics LLC.
    
      \n\n\n
  */
  public function getInstanceRecordsByPropertyValue($params, &$smarty_obj)
  {
    $instanceRecords = array();

    foreach ($this->instanceRecords as $resource => $propertiesValues)
    {
      if (is_array($propertiesValues[get_label_uri($params["property"])]))
      {
        foreach ($propertiesValues[get_label_uri($params["property"])] as $value)
        {
          if ($params["value"] == $value)
          {
            $instanceRecords[$resource] = $propertiesValues;
            break;
          }
        }
      }
    }

    if (count($instanceRecords) > 0)
    {
      // Sort the instanceRecords array by the first value of the target property
      if (isset($params["orderby"]))
      {
        // Case insensitive ordering
        foreach ($instanceRecords as $propertiesValues)
        {
          $sort_aux[] = strtolower($propertiesValues[get_label_uri($params["orderby"])][0]);
        }

        array_multisort($sort_aux, SORT_ASC, $instanceRecords);
      }
    }

    return ($instanceRecords);
  }
}

//@}


?>